local gen2d = require 'poisson'

local cmode="norm"

local lg=love.graphics
local lm=love.mouse

local taskBar={}



local function displaySingleDot() 
	for _,vert in ipairs(vertices) do
		love.graphics.setColor(HSL(vert.angle/(2*math.pi)*255, 255, 255*(vert.offset), 255))
		love.graphics.circle("fill", vert.x, vert.y, 5)
	end
end

local function displaySingleSquare() 
	for _,vert in ipairs(vertices) do
		love.graphics.setColor(HSL(vert.angle/(2*math.pi)*255, 255, 255*(vert.offset), 255))
		love.graphics.rectangle("fill", vert.x-5, vert.y-5, 10, 10)
	end
end

local function displayTwoDotsTransparent() 
	-- Color grid
	for _,vert in ipairs(vertices) do
			love.graphics.setColor(HSL(vert.angle/(2*math.pi)*255, 255, 255*(vert.offset), 128))
			love.graphics.circle("fill", vert.x, vert.y, 10)
	end
	for _,vert in ipairs(vertices) do
		love.graphics.setColor(HSL(vert.angle/(2*math.pi)*255, 255, 255*(vert.offset), 128))
		love.graphics.circle("fill", vert.x, vert.y, 5)
	end
end

local function displayTwoSquaresTransparent() 
	-- Color grid
	for _,vert in ipairs(vertices) do
			love.graphics.setColor(HSL(vert.angle/(2*math.pi)*255, 255, 255*(vert.offset), 128))
			love.graphics.rectangle("fill", vert.x-10, vert.y-10, 20,20)
	end
	for _,vert in ipairs(vertices) do
		love.graphics.setColor(HSL(vert.angle/(2*math.pi)*255, 255, 255*(vert.offset), 128))
		love.graphics.rectangle("fill", vert.x-5, vert.y-5, 10,10)
	end
end

local function displayTwoDotsSolid() 

	-- Color grid
	for _,vert in ipairs(vertices) do
			love.graphics.setColor(HSL(vert.angle/(2*math.pi)*255, 255, 255*(vert.offset), 255))
			love.graphics.circle("fill", vert.x, vert.y, 10)
	end
	for _,vert in ipairs(vertices) do
		love.graphics.setColor(HSL(vert.angle/(2*math.pi)*255, 255, 255*(vert.offset), 255))
		love.graphics.circle("fill", vert.x, vert.y, 5)
	end
end

local function displayTwoSquaresSolid() 
	-- Color grid
	for _,vert in ipairs(vertices) do
			love.graphics.setColor(HSL(vert.angle/(2*math.pi)*255, 255, 255*(vert.offset), 255))
			love.graphics.rectangle("fill", vert.x-10, vert.y-10, 20,20)
	end
	for _,vert in ipairs(vertices) do
		love.graphics.setColor(HSL(vert.angle/(2*math.pi)*255, 255, 255*(vert.offset), 255))
		love.graphics.rectangle("fill", vert.x-5, vert.y-5, 10,10)
	end
end



local idMode=1
local displayModes = {
	{title="Single dot", f=displaySingleDot},
	{title="Two dots, transparent",f=displayTwoDotsTransparent},
	{title="Two dots, solid",f=displayTwoDotsSolid},
	{title="Single square",f=displaySingleSquare},
	{title="Two squares, transparent",f=displayTwoSquaresTransparent},
	{title="Two squares, solid",f=displayTwoSquaresSolid}
}


local function oneOctave()
	for _,vert in ipairs(vertices) do
		local xx,yy,zz,zz1=vert.x/200,vert.y/200, cur_time/2, cur_time/2-20
		local off,ang = love.math.noise(xx,yy,zz),love.math.noise(xx,yy,zz1)
		vert.offset = off
		vert.angle = ang*2*math.pi
	end
end

local function twoOctaves()
	for _,vert in ipairs(vertices) do
		local xx,yy,zz,zz1=vert.x/200,vert.y/200, cur_time/2, cur_time/2-20
		local off = (love.math.noise(xx,yy,zz) + love.math.noise(xx*2,yy*2,zz*2)/2)/1.5
		local ang = (love.math.noise(xx,yy,zz1) + love.math.noise(xx*2,yy*2,zz1*2)/2)/1.5
		vert.offset = off
		vert.angle = ang*2*math.pi
	end
end

local function threeOctaves()
	for _,vert in ipairs(vertices) do
		local xx,yy,zz,zz1=vert.x/200,vert.y/200, cur_time/2, cur_time/2-20
		local off = (love.math.noise(xx,yy,zz) + love.math.noise(xx*2,yy*2,zz*2)/2 + love.math.noise(xx*4,yy*4,zz*4)/4)/1.75
		local ang = (love.math.noise(xx,yy,zz1) + love.math.noise(xx*2,yy*2,zz1*2)/2 + love.math.noise(xx*4,yy*4,zz1*4)/4)/1.75
		vert.offset = off
		vert.angle = ang*2*math.pi
	end
end

local function fourOctaves()
	for _,vert in ipairs(vertices) do
		local xx,yy,zz,zz1=vert.x/200,vert.y/200, cur_time/2, cur_time/2-20
		local off = (love.math.noise(xx,yy,zz) + love.math.noise(xx*2,yy*2,zz*2)/2 + love.math.noise(xx*4,yy*4,zz*4)/4+ love.math.noise(xx*8,yy*8,zz*8)/8)/1.875
		local ang = (love.math.noise(xx,yy,zz1) + love.math.noise(xx*2,yy*2,zz1*2)/2 + love.math.noise(xx*4,yy*4,zz1*4)/4+ love.math.noise(xx*8,yy*8,zz1*8)/8)/1.875
		vert.offset = off
		vert.angle = ang*2*math.pi
	end
end

local nbOctaves=1
local octaves = {oneOctave,twoOctaves,threeOctaves,fourOctaves}
	
local function initTaskBar(f)
	taskBar.font   = f
	taskBar.height =  f:getHeight() + 15
	taskBar.bg = {0,0,196,196}
	taskBar.fg = {255,255,255,255}
	taskBar.fgi = {0,0,255,255}
	
	taskBar.triggerY = 5
	taskBar.visible=false
	taskBar.Y = 0
	taskBar.Yspeed = 100
	taskBar.Ytarget=taskBar.height
	taskBar.dY=0
	
	taskBar.text1="# OCTAVES : "
	taskBar.xtext1=20
	taskBar.ytext1=math.floor(taskBar.height/2 - f:getHeight()/2)
	
	taskBar.boxH=f:getHeight()+6
	taskBar.boxW=f:getWidth("M")+6
	taskBar.boxY=math.floor(taskBar.height/2 - taskBar.boxH/2)
	local boxX=f:getWidth(taskBar.text1) + taskBar.xtext1
	taskBar.boxX={}
	taskBar.boxX[1]=boxX
	for i=2,#octaves do
		taskBar.boxX[i]=taskBar.boxX[i-1]+taskBar.boxW+5
	end

	taskBar.text2="DISPLAY MODE : "
	taskBar.xtext2=taskBar.boxX[#taskBar.boxX]+taskBar.boxW+10
	taskBar.ytext2=math.floor(taskBar.height/2 - f:getHeight()/2)
	
	local maxW=0
	for _,dm in ipairs(displayModes) do
		local w=f:getWidth(dm.title)
		if w > maxW then maxW = w end
	end
		
	taskBar.choiceH = f:getHeight()+6
	taskBar.choiceW = maxW +6
	taskBar.choiceX = taskBar.xtext2 + f:getWidth(taskBar.text2)
	taskBar.choiceY = math.floor(taskBar.height/2 - taskBar.choiceH/2)
end


function taskBar:draw()
	if self.visible then
		lg.setColor(self.bg)
		lg.rectangle('fill',0,self.Y-self.height,stage.w,self.height)
		lg.setFont(self.font)
		lg.setColor(self.fg)
		lg.print(self.text1,self.xtext1,self.ytext1+self.Y-self.height)
		for i=1,#self.boxX do
			lg.setColor(self.fg)
			lg.rectangle((nbOctaves==i and 'fill') or 'line',self.boxX[i],self.boxY+self.Y-self.height,self.boxW,self.boxH)			
			if (nbOctaves==i) then lg.setColor(self.fgi) end
			lg.printf(i,self.boxX[i]+3,self.boxY+self.Y-self.height+3,self.boxW-6,'center')
		end
		lg.setColor(self.fg)
		lg.print(self.text2,self.xtext2,self.ytext2+self.Y-self.height)
		lg.rectangle('line',self.choiceX,self.choiceY+self.Y-self.height,self.choiceW,self.choiceH)
		lg.printf(displayModes[idMode].title,self.choiceX+3,self.choiceY+3+self.Y-self.height,self.choiceW-6,'left')
	end
end

function taskBar:update(dt)
	if not self.visible and lm.getY() <= self.triggerY then 
		self.visible = true
		self.dY=self.Yspeed
		self.Y=self.triggerY+1
	elseif self.visible then
		if lm.getY()>self.Y then
			self.dY=-self.Yspeed
		elseif self.dY ~= 0 then
			self.dY = self.Yspeed
		end
		if dY ~= 0 then
			self.Y = self.Y + dt * self.dY
			if self.Y >= self.Ytarget then
				self.Y = self.Ytarget
				self.dY=0
			elseif self.Y<0 then
				self.dY=0
				self.visible=false
				self.Y = 0
			end
		end
	end
end

function taskBar:click(x,y)
	if x>=self.choiceX and x <= self.choiceX+self.choiceW then
		nextDisplayMode()
	end

	local ux = x-self.boxW
	if x>=self.boxX[1] and ux<=self.boxX[#self.boxX] then
		for i=1,#self.boxX do
			if x>=self.boxX[i] and ux <= self.boxX[i] then
				nbOctaves=i
				break
			end
		end
	end
end

function love.load()
	stage = {w=lg.getWidth(), h=lg.getHeight()}
	initTaskBar(lg.newFont(18))
	spacing = 10

	local points=gen2d(stage.w-20,stage.h-20,10)
	
	vertices = {}
	for _,p in ipairs(points) do
		table.insert(vertices,{offset=0,angle=0,x=math.floor(p.x)+10,y=math.floor(p.y)+10})
	end

	cur_time = 0
end

function nextDisplayMode()
	idMode = idMode+1
	if idMode > #displayModes then
		idMode=1
	end
end

function love.keypressed(key)
	if key == "tab" then
		nextDisplayMode()
	end
end

function love.mousepressed( x, y, button ) 
	--nextDisplayMode()
	if taskBar.visible and y < taskBar.Y then
		taskBar:click(x,y)
	end

end
function love.update(dt)
	cur_time = cur_time + dt
	octaves[nbOctaves]()
	taskBar:update(dt)
end

-- Converts HSL to RGB. (input and output range: 0 - 255)
function HSL(h, s, l, a)
    if s<=0 then return l,l,l,a end
    h, s, l = h/256*6, s/255, l/255
    local c = (1-math.abs(2*l-1))*s
    local x = (1-math.abs(h%2-1))*c
    local m,r,g,b = (l-.5*c), 0,0,0
    if h < 1     then r,g,b = c,x,0
    elseif h < 2 then r,g,b = x,c,0
    elseif h < 3 then r,g,b = 0,c,x
    elseif h < 4 then r,g,b = 0,x,c
    elseif h < 5 then r,g,b = x,0,c
    else              r,g,b = c,0,x
    end return (r+m)*255,(g+m)*255,(b+m)*255,a
end

function love.draw()
	love.graphics.setColor(0, 0, 0)
	love.graphics.rectangle("fill", 0, 0, stage.w, stage.h)
	love.graphics.setColor(255, 255, 255)
	displayModes[idMode].f()
	taskBar:draw()

end